#include "ipv4_adr.h"
#include "general.h"
#include <iostream>
#include <boost/regex.hpp>

ipv4_adr::ipv4_adr(const string &a)
{
    boost::regex rx("(\\d+)\\.(\\d+)\\.(\\d+)\\.(\\d+)");
    boost::smatch s;
    if (boost::regex_match(a, s, rx)) {
        adr = 0;
        for (int i=1; i<=4; ++i) {
            int b = lexical_cast<U32>(s[i]);
            if (b>255) {
                throw std::ios_base::failure("IPv4 address component > 255");
            }
            adr <<= 8;
            adr += b;
        }
    } else {
        throw std::ios_base::failure("Syntax error in IPv4 address");
    }
    
}

ipv4_adr::ipv4_adr(struct in_addr *a)
{
    adr = ntohl(*(unsigned int *)a);
}

int ipv4_adr::operator[](int index) const
{
    switch (index) {
    case 3:
        return adr & 0xff;
    case 2:
        return (adr>>8) & 0xff;        
    case 1:
        return (adr>>16) & 0xff;
    case 0:
        return (adr>>24) & 0xff;
    default:
        return 0;                    
    }
}

string ipv4_adr::str() const
{
    string result;
    for (int i=0; i<4; ++i) {
        if (i) {
            result += ".";
        }
        result += lexical_cast<string>(operator[](i));
    }
    return result;
}

std::ostream& operator<<(std::ostream &str, const ipv4_adr &adr)
{
    str << adr[0] << "." << adr[1] << "." << adr[2] << "." << adr[3];
    return str;
}

std::istream& operator>>(std::istream &str, ipv4_adr &adr)
{
    string stradr;
    str >> stradr;
    try {
        adr = ipv4_adr(stradr);
    } catch (std::ios_base::failure &) {    
        str.setstate(std::ios::failbit);
    }
    return str;
}
