#ifndef __IPV4_ADR
#define __IPV4_ADR

#include <iostream>
#include <string>
#ifdef WIN32
#include "winsock.h"
#else
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#endif

using std::string;

class ipv4_adr
{
private:
    unsigned int adr;
public:
    ipv4_adr() : adr(0) { };
    explicit ipv4_adr(unsigned int a) : adr(a) { };
    explicit ipv4_adr(const string &a);
    explicit ipv4_adr(struct in_addr *a);
    ipv4_adr(int w, int x, int y, int z) : adr((((((w<<8)+x)<<8)+y)<<8)+z) { };
    ipv4_adr(const ipv4_adr &other) : adr(other.adr) { };
    ipv4_adr &operator=(const ipv4_adr &other) { adr=other.adr; return *this; };
    unsigned int get() const { return adr; };
    unsigned int get_n() const { return htonl(adr); };
    bool empty() const { return adr==0; };
    operator bool() const { return adr!=0; };
    int operator[](int index) const;          
    bool operator<(const ipv4_adr &other) const { return adr<other.adr; };
    bool operator<=(const ipv4_adr &other) const { return adr<=other.adr; };
    bool operator==(const ipv4_adr &other) const { return adr==other.adr; };
    bool operator!=(const ipv4_adr &other) const { return adr!=other.adr; };
    bool operator>(const ipv4_adr &other) const { return adr>other.adr; };
    bool operator>=(const ipv4_adr &other) const { return adr>=other.adr; };
    ipv4_adr operator+(unsigned int delta) const { return ipv4_adr(adr+delta); };
    void set_tail(int prefix_length, unsigned int tail) { adr = (adr & ~((1<<(32-prefix_length))-1)) + tail; };
    string str() const;
    static string parse_error_string() { return "IPv4 address in a.b.c.d form expected"; };
    template<class Archive>
    void serialize(Archive & ar, const unsigned int version)
    {
        ar & adr;
    }
};


std::ostream& operator<<(std::ostream &str, const ipv4_adr &adr);
std::istream& operator>>(std::istream &str, ipv4_adr &adr);

#endif
