#include "ipv4_host.h"
#include "general.h"
#include <iostream>
#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>

ipv4_host::ipv4_host(const string &h)
{
    if (h.find_first_not_of(".0123456789")==string::npos) {
        host_adr = ipv4_adr(h);
    } else {
        host_name = h;
        refresh();
    }
}

string ipv4_host::str() const 
{
    if (host_name.size()) {
        return host_name;
    } else {
        return host_adr.str();
    }   
}

string ipv4_host::str_with_adr() const
{
    string result;
    if (host_name.size()) {
        result = host_name;
        result += " (";
        result += host_adr.str();
        result += ")";
    } else {
        result = host_adr.str();
    }
    return result;
}

const ipv4_adr &ipv4_host::refresh()
{
    if (host_name.size()) {
        struct hostent *hp = ::gethostbyname(host_name.c_str());
        if (hp && hp->h_addr_list[0]) {
            host_adr = ipv4_adr((in_addr*)&(hp->h_addr_list[0]));
        } else {
            host_adr = ipv4_adr(0,0,0,0);
        }
    }
    return host_adr;
}

std::ostream& operator<<(std::ostream &str, const ipv4_host &host)
{
    str << host.str();
    return str;
}

std::istream& operator>>(std::istream &str, ipv4_host &host)
{
    string strhost;
    str >> strhost;
    try {
        host = ipv4_host(strhost);
    } catch (std::ios_base::failure &) {    
        str.setstate(std::ios::failbit);
    }
    return str;
}

