#include "general.h"
#include "ipv4_host_port.h"
#include <boost/regex.hpp>

ipv4_host_port::ipv4_host_port(const string &a, U32 dflt/*=0xffffffff*/)
{
    boost::regex rx("([^:]+)(:(\\d+))?");
    boost::smatch s;
    if (boost::regex_match(a, s, rx) && (s[2].matched || dflt!=0xffffffff)) {
        host = ipv4_host(s[1].str());
        if (s[2].matched) {
            port = lexical_cast<U16>(s[3].str());
        } else {
            port = dflt;
        }
    } else {
        throw std::ios_base::failure("Syntax error in IPv4 host:port");
    }
    
}

std::ostream& operator<<(std::ostream &str, const ipv4_host_port &sn)
{
    str << sn.get_host() << ":" << sn.get_port();
    return str;
}

std::istream& operator>>(std::istream &str, ipv4_host_port &sn)
{
    string strsn;
    str >> strsn;
    try {
        sn = ipv4_host_port(strsn);
    } catch (std::ios_base::failure &) {    
        str.setstate(std::ios::failbit);
    }
    return str;
}

std::ostream& operator<<(std::ostream &str, const ipv4_host_opt_port &sn)
{
    ipv4_host_port hp(sn);
    str << hp;
    return str;
}

std::istream& operator>>(std::istream &str, ipv4_host_opt_port &sn)
{
    string strsn;
    str >> strsn;
    try {
        sn = ipv4_host_port(strsn, 0);
    } catch (std::ios_base::failure &) {    
        str.setstate(std::ios::failbit);
    }
    return str;
}


