#ifndef __IPV4_HOST_PORT
#define __IPV4_HOST_PORT

//#include "general.h"
#include "ipv4_host.h"
#include <iostream>

typedef unsigned int U32;

class ipv4_host_port
{
protected:
    ipv4_host host;
    U32 port;
public:
    ipv4_host_port() : host(), port(0) { };
    ipv4_host_port(const ipv4_host_port &other) : host(other.host), port(other.port) { };
    ipv4_host_port(const ipv4_host &h, U32 p) : host(h), port(p) { };
    ipv4_host_port(const ipv4_adr &a, U32 p) : host(a), port(p) { };
    explicit ipv4_host_port(const string &sn, U32 dflt=0xffffffff);
    ipv4_host_port &operator=(const ipv4_host_port &other) 
        { host = other.host; port = other.port; return *this; };
    const ipv4_host &get_host() const { return host; };
    const ipv4_adr &get_adr() const { return host.get_adr(); };
    const string &get_name() const { return host.get_name(); };
    U32 get_port() const { return port; };
    bool operator<(const ipv4_host_port &other) const
        { return host.get_adr()<other.host.get_adr() || (host.get_adr()==other.host.get_adr() && port<other.port); };
    bool less(const ipv4_host_port &other) const
        { return *this<other; };
    bool operator==(const ipv4_host_port &other) const
        { return host.get_adr()==other.host.get_adr() && port==other.port; };
    bool operator!=(const ipv4_host_port &other) const
        { return !(*this==other); };
    static string parse_error_string() { return "host name or address followed by ':port-number' expected"; };
};

class ipv4_host_opt_port : public ipv4_host_port
{
public:
    ipv4_host_opt_port() { };
    ipv4_host_opt_port(const ipv4_host_opt_port &other) : ipv4_host_port(other) { };
    ipv4_host_opt_port(const ipv4_host_port &other) : ipv4_host_port(other) { };
    explicit ipv4_host_opt_port(const string &sn) : ipv4_host_port(sn, 0) { };
    ipv4_host_opt_port &operator=(const ipv4_host_port &other) 
        { *(ipv4_host_port*)this = other; return *this; };
    static string parse_error_string() { return "host name or address optionally followed by ':port-number' expected"; };
};

std::ostream& operator<<(std::ostream &str, const ipv4_host_port &sn);
std::istream& operator>>(std::istream &str, ipv4_host_port &sn);
std::ostream& operator<<(std::ostream &str, const ipv4_host_opt_port &sn);
std::istream& operator>>(std::istream &str, ipv4_host_opt_port &sn);

#endif
