#ifndef __SOCKET
#define __SOCKET

#ifdef WIN32
#include "winsock.h"
#else
#include <sys/types.h>
#include <sys/socket.h>
#define SOCKET int
#endif

#include <vector>
#include "ipv4_adr.h"
#include "ipv4_host_port.h"
using std::vector;

class socket
{
private:
    SOCKET id;
public:
    ~socket();
    SOCKET get() const { return id; };
    void listen();
    int send(const char *buffer, int len);
    template<class T> int send(const T& buffer);
    int sendto(const char *buffer, int len, struct sockaddr *adr, int adrlen);
    template<class T> int sendto(const T& buffer, struct sockaddr *adr, int adrlen);
    int recv(char *buffer, int len);
    int recvfrom(char *buffer, int len, struct sockaddr *adr, int *adrlen);
    void close();
protected:
    socket(int domain, int type);
    void bind(struct sockaddr *adr, int len);
    void accept(socket *sock, struct sockaddr *adr, int *adrlen);
    void connect(struct sockaddr *adr, int len);
    static string strerror_s(int e);
};

class ip_socket : public socket
{
public:
    struct peer_info
    {
        ipv4_adr adr;
        unsigned int port;
        peer_info() : adr((unsigned int)INADDR_ANY), port(0) { };
        peer_info(unsigned int a, unsigned int p) : adr(a), port(p) { };
    };
public:    
    ip_socket(int type=SOCK_STREAM);
    void bind(unsigned int port, unsigned int ipadr=INADDR_ANY);
    void accept(ip_socket &sock, peer_info &pi);
    static string gethostname(ipv4_adr adr);
    static void gethostadr(vector<ipv4_adr> &result, const string &name);
};

class tcp_socket : public ip_socket
{
public:
    tcp_socket();
    void connect(const ipv4_adr &adr, unsigned int port);
    void connect(const string &host, unsigned int port);
};

class udp_socket : public ip_socket
{
public:
    udp_socket();
    template<class T> int sendto(const T& buffer, unsigned int ipadr, unsigned int port);
    int recvfrom(char *buffer, int len, peer_info &pi);
    int sendto(const char *buffer, int len, const ipv4_adr &adr, unsigned int port);
    int sendto(const char *buffer, int len, const ipv4_host_port &target)
        {
            return sendto(buffer, len, target.get_adr(), target.get_port());
        }       
};

#endif
